#include "Camera.h"
#include <GL/glut.h>
#include <GL/glext.h>

Camera::Camera(){
	eye[0]		= 0.0f;
	eye[1]		= 30.0f;
	eye[2]		= 100.0f;
	eye[3]		= 1.0f;
	lookAt[0]	= 0.0f;
	lookAt[1]	= 0.0f;
	lookAt[2]	= -1.0f;
	lookAt[3]	= 1.0f;
	viewUp[0]	= 0.0f;
	viewUp[1]	= 1.0f;
	viewUp[2]	= 0.0f;
	viewUp[3]	= 1.0f;
	moveState	= 0x0000;
	groundMove	= false;
	xzAngle		= 90.0f;
	incX		= .5f;
	incY		= .005f;
	speed		= 1.0f;
}

Camera::~Camera(){};

void Camera::rotate(float dX, float dY){
	xzAngle += incX * dX;
	lookAt[1] += incY * dY;
}

void Camera::update(){
	float x = cos(PI * xzAngle/180.0f);
	float z = -sin(PI * xzAngle/180.0f);

	if(lookAt[1] >= 1.0f)
		lookAt[1] = .999f;
	if(lookAt[1] <= -1.0f)
		lookAt[1] = -.999f;

	lookAt[0] = x * (1 - fabs(lookAt[1]));
	lookAt[2] = z * (1 - fabs(lookAt[1]));

	if(groundMove){
		if(moveState & CAMERA_FORWARD){
			eye[0] += lookAt[0]*speed;
			eye[2] += lookAt[2]*speed;
		}
		if(moveState & CAMERA_BACK){
			eye[0] -= lookAt[0]*speed;
			eye[2] -= lookAt[2]*speed;
		}
	}
	else{
		if(moveState & CAMERA_FORWARD)	
			eye += lookAt * speed;
		if(moveState & CAMERA_BACK)		
			eye -= lookAt * speed;
	}
	if(moveState & CAMERA_LEFT){
		eye[0] += z * speed;
		eye[2] -= x * speed;
	}
	if(moveState & CAMERA_RIGHT){
		eye[0] -= z * speed;
		eye[2] += x * speed;
	}
	if(moveState & CAMERA_UP)
		eye[1] += speed;
	if(moveState & CAMERA_DOWN)
		eye[1] -= speed;

	xzAngle = fmodf(xzAngle, 360.0f);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt(eye[0], eye[1], eye[2], 
			  eye[0] + lookAt[0], eye[1] + lookAt[1], eye[2] + lookAt[2], 
			  viewUp[0], viewUp[1], viewUp[2]);
}