 //OBJ_Loader.cpp : Defines the entry point for the console application.

#include <iostream>
#include <math.h>

#include "defines.h"
#include "vec4.h"

#include <GL/glew.h>

#include "Camera.h"
#include "glm.h"

#include "shader.h"

#include <GL/glut.h>
#include <GL/glext.h>

#include <string.h>
#include <tchar.h>

using namespace std;

//#define USE_SKYS_SHADER
#define USE_SKIN_SHADER
#define USE_HATCH_SHADER
//#define USE_HATCH_SHADER2
#define USE_WATERCOLOR_SHADER
//#define USE_WATERCOLOR_SHADER2

struct Window{
	GLint w, h;		// width and height of window
	GLfloat aspectRatio(){return (GLfloat)w/(GLfloat)h;}
};

Camera *camera = new Camera();
GLMmodel *model;
Window window = {800, 600};
int update_tick = 0;

bool bLightOn = true;
//GLfloat GlobalAmbient[]= { 0.0f, 0.0f, 0.0f, 1.0f };
GLfloat LightAmbient[]= { 0.6f, 0.5f, 0.5f, 1.0f }; 		
GLfloat LightDiffuse[]= { 0.9f, 0.9f, 0.9f, 1.0f };	
GLfloat LightSpecular[]= { 1.0f, 1.0f, 1.0f, 1.0f };	
GLfloat LightPosition[]= { 500.0f, 1000.0f, -1500.0f, 1.0f };


enum {
#ifdef USE_SKYS_SHADER
	SKYS_SHADER,
#endif

#ifdef USE_SKIN_SHADER
	SKIN_SHADER,
#endif
#ifdef USE_WATERCOLOR_SHADER	
	WATERCOLOR_SHADER,
#endif
#ifdef USE_WATERCOLOR_SHADER2	
	WATERCOLOR_SHADER2,
#endif
#ifdef USE_HATCH_SHADER	
	HATCH_SHADER,
#endif
#ifdef USE_HATCH_SHADER2
	HATCH_SHADER2,
#endif
	SHADER_COUNT
};

GLuint activeProg, loadedProg[SHADER_COUNT];

bool initialize();
void render();
void keyboardFunc(unsigned char key, int mx, int my);
void keyboardUpFunc(unsigned char key, int mx, int my);
void mouseFunc(int button, int state, int x, int y);
void motionFunc(int x, int y);
void cleanup();

using namespace std;

void getGlVersion( int *major, int *minor )
{
    const char* verstr = (const char*)glGetString( GL_VERSION );
    if( (verstr == NULL) || (sscanf( verstr, "%d.%d", major, minor ) != 2) )
    {
        *major = *minor = 0;
        fprintf( stderr, "Invalid GL_VERSION format!!!\n" );
    }
}



int main(int argc, char* argv[])
{

	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH);
	glutInitWindowPosition(100,100);
	glutInitWindowSize(800,600);
	glutCreateWindow("CS 580 Project - Obj Loader");
	//glutSetCursor(GLUT_CURSOR_NONE);
	glutIgnoreKeyRepeat(1);

	glutDisplayFunc(render);

	glutKeyboardFunc(keyboardFunc);
	glutKeyboardUpFunc(keyboardUpFunc);
	glutPassiveMotionFunc(motionFunc);
	glutMotionFunc(motionFunc);
	glutMouseFunc(mouseFunc);

	bool success = initialize();

	if (success) {
		update_tick = glutGet(GLUT_ELAPSED_TIME) + 10;
		glutMainLoop();
	} else {
		char wait;
		std::cin >> wait;
	}

	return 0;
}

bool initialize() {

	//vector<string> filenames(NUM_TEXTURES); 

	glClearColor(0.2f, 0.1f, 0.05f, 1.0f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	//glActiveTexture = (PFNGLACTIVETEXTUREPROC) wglGetProcAddress ("glActiveTexture");

	glEnable(GL_CULL_FACE);
	glCullFace(GL_BACK);
	glFrontFace(GL_CCW);
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);

	glEnable(GL_LIGHTING);
	glLightfv(GL_LIGHT0, GL_POSITION, LightPosition);
	glLightfv(GL_LIGHT0, GL_AMBIENT, LightAmbient);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, LightDiffuse);
	glLightfv(GL_LIGHT0, GL_SPECULAR, LightSpecular);
	glEnable(GL_LIGHT0);

	//glLightModelfv(GL_LIGHT_MODEL_AMBIENT, GlobalAmbient);
	
	glEnable(GL_TEXTURE_2D);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(60, window.aspectRatio(), 1.0, 5000.0);
	
	
	int gl_major, gl_minor;
	getGlVersion(&gl_major, &gl_minor);
	printf("GL_VERSION major=%d minor=%d\n", gl_major, gl_minor);

	if (gl_major < 2)
	{
		printf("GL_VERSION major=%d minor=%d\n", gl_major, gl_minor);
		printf("Support for OpenGL 2.0 is required for this demo...exiting\n");
		return false;
	} 
	GLchar *VertexShaderSource, *FragmentShaderSource;
	glewInit();

	for (int i = 0; i < SHADER_COUNT; i ++)
		loadedProg[i] = 0;

	#ifdef USE_SKYS_SHADER
	{
		cout << "Loading Skys shader, press "<<SKYS_SHADER+1<<" to activate\n";
		GLuint skyProg;				

		if (!readShaderSource("../shaders/skys", &VertexShaderSource, &FragmentShaderSource)) return false;	
		if (!installShaders(VertexShaderSource, FragmentShaderSource, skyProg)) return false;

		glUniform1i(glGetUniformLocation(skyProg,"modelTex"),0);

		//glUniform4f(glGetUniformLocation(skyProg, "ambient"), 0.1, 0.1, 0.1, 1.0);

		activeProg = loadedProg[SKYS_SHADER] = skyProg;
	}
	#endif
	#ifdef USE_SKIN_SHADER
	{
		cout << "Loading Skin shader, press "<<SKIN_SHADER+1<<" to activate\n";
		GLuint skinProg;
		GLuint skinTex;		
		string skinFilename = "../models/mat/skin.jpg";
		
		if (!loadShaderTex((wchar_t*)skinFilename.c_str(),skinTex)) return false;

		if (!readShaderSource("../shaders/skin", &VertexShaderSource, &FragmentShaderSource)) return false;		
		if (!installShaders(VertexShaderSource, FragmentShaderSource, skinProg)) return false;
		
		glUniform1i(glGetUniformLocation(skinProg,"skinTex"),skinTex);
		glUniform1i(glGetUniformLocation(skinProg,"modelTex"),0);

		glUniform4f(glGetUniformLocation(skinProg, "ambient"), 0.1, 0.1, 0.1, 1.0);
		glUniform4f(glGetUniformLocation(skinProg, "lightColor"), 0.8, 0.8, 0.8, 1.0);
		glUniform1f(glGetUniformLocation(skinProg, "kRim"), 2.0);
		glUniform1f(glGetUniformLocation(skinProg, "kR"), 0.75);
		glUniform1f(glGetUniformLocation(skinProg, "kS"), 0.82);
		glUniform1f(glGetUniformLocation(skinProg, "kSpec"), 5.0);
		glUniform1f(glGetUniformLocation(skinProg, "albedo"), 0.8);
		glUniform1f(glGetUniformLocation(skinProg, "fresnelS"), 0.63);

		 loadedProg[SKIN_SHADER] = skinProg;
	}
	#endif

	
	#if defined(USE_HATCH_SHADER) || defined(USE_HATCH_SHADER2)
	{
		cout << "Loading Hatch shader, press "<<HATCH_SHADER+1<<" to activate.\n";
		GLuint hatchProg;
		string hatchFilenames[] = 
		{
			"../models/mat/Hatch0.jpg",
			"../models/mat/Hatch1.jpg",
			"../models/mat/Hatch2.jpg",
			"../models/mat/Hatch3.jpg",
			"../models/mat/Hatch4.jpg",
			"../models/mat/Hatch5.jpg"
		};		
		GLuint hatchTex[6];
		
		int num_tex = 6;
		//glGenTextures(num_tex, hatchTex);
		for (int i = 0; i < num_tex; i++)	{
			if (!loadShaderTex((const wchar_t*)(hatchFilenames[i].c_str()), hatchTex[i]))
				return false;
			printOpenGLError();
		}
		
		#ifdef USE_HATCH_SHADER
			
				if (!readShaderSource("../shaders/hatch", &VertexShaderSource, &FragmentShaderSource))
					return false;
				printOpenGLError();
				if (!installShaders(VertexShaderSource, FragmentShaderSource, hatchProg))
					return false;

				glUniform1i(glGetUniformLocation(hatchProg,"Hatch0"),hatchTex[0]);
				glUniform1i(glGetUniformLocation(hatchProg,"Hatch1"),hatchTex[1]);
				glUniform1i(glGetUniformLocation(hatchProg,"Hatch2"),hatchTex[2]);
				glUniform1i(glGetUniformLocation(hatchProg,"Hatch3"),hatchTex[3]);
				glUniform1i(glGetUniformLocation(hatchProg,"Hatch4"),hatchTex[4]);
				glUniform1i(glGetUniformLocation(hatchProg,"Hatch5"),hatchTex[5]);
				glUniform1i(glGetUniformLocation(hatchProg,"ModelTex"),0);
				glUniform2f(glGetUniformLocation(hatchProg, "fViewportDimensions"), 800.0f, 600.0f);

				loadedProg[HATCH_SHADER] = hatchProg;
		#endif
		#ifdef USE_HATCH_SHADER2
				if (!readShaderSource("../shaders/hatch2", &VertexShaderSource, &FragmentShaderSource))
					return false;
				printOpenGLError();
				if (!installShaders(VertexShaderSource, FragmentShaderSource, hatchProg))
					return false;

				glUniform1i(glGetUniformLocation(hatchProg,"Hatch0"),hatchTex[0]);
				glUniform1i(glGetUniformLocation(hatchProg,"Hatch1"),hatchTex[1]);
				glUniform1i(glGetUniformLocation(hatchProg,"Hatch2"),hatchTex[2]);
				glUniform1i(glGetUniformLocation(hatchProg,"Hatch3"),hatchTex[3]);
				glUniform1i(glGetUniformLocation(hatchProg,"Hatch4"),hatchTex[4]);
				glUniform1i(glGetUniformLocation(hatchProg,"Hatch5"),hatchTex[5]);
				glUniform1i(glGetUniformLocation(hatchProg,"ModelTex"),0);
				glUniform2f(glGetUniformLocation(hatchProg, "fViewportDimensions"), 800.0f, 600.0f);

				loadedProg[HATCH_SHADER2] = hatchProg;		
		#endif
		}
	#endif

	#if defined(USE_WATERCOLOR_SHADER) || defined(USE_WATERCOLOR_SHADER2)
	{
		cout << "Loading Watercolor shader, press "<<WATERCOLOR_SHADER+1<<" to activate\n";
		GLuint waterColorProg;
		string watercolorFilenames[] = {
			"../models/mat/colormap.png",
			"../models/mat/noise.tga",
			"../models/mat/colormap2.png"
		};
		GLuint watercolorTex[3];

		int num_tex = 3;
		//glGenTextures(num_tex, watercolorTex);
		for (int i = 0; i < num_tex; i++)	{
			if (!loadShaderTex((const wchar_t*)watercolorFilenames[i].c_str(), watercolorTex[i]))
				return false;
			printOpenGLError();
		}

		#ifdef USE_WATERCOLOR_SHADER
				if(!readShaderSource("../shaders/watercolor", &VertexShaderSource, &FragmentShaderSource))
					return false;
				if(!installShaders(VertexShaderSource, FragmentShaderSource, waterColorProg))
					return false;

				glUniform1i(glGetUniformLocation(waterColorProg,"colorTex"),watercolorTex[0]);
				glUniform1i(glGetUniformLocation(waterColorProg,"paperTex"),watercolorTex[1]);
				glUniform1i(glGetUniformLocation(waterColorProg,"granTex"),watercolorTex[2]);
				glUniform1i(glGetUniformLocation(waterColorProg,"sceneTex"),0);
				glUniform2f(glGetUniformLocation(waterColorProg, "fViewportDimensions"), 800.0f, 600.0f);
				glUniform1f(glGetUniformLocation(waterColorProg, "M"), .40f);
				glUniform1f(glGetUniformLocation(waterColorProg, "D"), .50f);
				glUniform1f(glGetUniformLocation(waterColorProg, "G"), .50f);
				glUniform1f(glGetUniformLocation(waterColorProg, "P"), .50f);
				glUniform1f(glGetUniformLocation(waterColorProg, "S"), .30f);

				loadedProg[WATERCOLOR_SHADER] = waterColorProg;
		#endif
		#ifdef USE_WATERCOLOR_SHADER2
				if(!readShaderSource("../shaders/watercolor2", &VertexShaderSource, &FragmentShaderSource))
					return false;
				if(!installShaders(VertexShaderSource, FragmentShaderSource, waterColorProg))
					return false;

				glUniform1i(glGetUniformLocation(waterColorProg,"colorTex"),watercolorTex[0]);
				glUniform1i(glGetUniformLocation(waterColorProg,"paperTex"),watercolorTex[1]);
				glUniform1i(glGetUniformLocation(waterColorProg,"granTex"),watercolorTex[2]);
				glUniform1i(glGetUniformLocation(waterColorProg,"sceneTex"),0);
				glUniform2f(glGetUniformLocation(waterColorProg, "fViewportDimensions"), 800.0f, 600.0f);
				glUniform1f(glGetUniformLocation(waterColorProg, "M"), .40f);
				glUniform1f(glGetUniformLocation(waterColorProg, "D"), .50f);
				glUniform1f(glGetUniformLocation(waterColorProg, "G"), .50f);
				glUniform1f(glGetUniformLocation(waterColorProg, "P"), .50f);
				glUniform1f(glGetUniformLocation(waterColorProg, "S"), .30f);

				loadedProg[WATERCOLOR_SHADER2] = waterColorProg;
		#endif
	}
	#endif

	cout << "Done loading shaders.\n";

	glActiveTexture(GL_TEXTURE0);
	model = glmReadOBJ("../models/spy_scene_extended.obj");
	glmVertexNormals(model, 180.0, false);
	
    return true;

}

int count = 0;

void render() {	
	int starttime = glutGet(GLUT_ELAPSED_TIME);
	while(glutGet(GLUT_ELAPSED_TIME) > update_tick) {
		camera->update();
		update_tick += 10;
	}	

	camera->update();

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt(camera->eye[0], camera->eye[1], camera->eye[2], 
			  camera->eye[0] + camera->lookAt[0], 
			  camera->eye[1] + camera->lookAt[1], 
			  camera->eye[2] + camera->lookAt[2], 
			  camera->viewUp[0], camera->viewUp[1], camera->viewUp[2]);

	glLightfv(GL_LIGHT0, GL_POSITION, LightPosition);	

	glUseProgram(activeProg);
	glPushMatrix();		
	glScalef(0.5f,0.5f,0.5f);	
	glmDraw(model, GLM_SMOOTH | GLM_TEXTURE | GLM_MATERIAL );	
	glPopMatrix();

	glutSwapBuffers();
	glutPostRedisplay();

	float fps = 1000.0f / (float)(glutGet(GLUT_ELAPSED_TIME) - starttime);
	//printf("FPS is %f\n",fps);
}

void keyboardFunc(unsigned char key, int mx, int my){
	switch(key){
		case '1':case '2':case '3': case '4': case '5': case '6': case '7': case '8': case '9': case '0':
			{
				int i = key -48;				
				if (i <= 0 || i > SHADER_COUNT)
					activeProg = 0;
				else
					activeProg = loadedProg[i-1];
			}
			break;
		case 'l': case 'L':
			{	bLightOn = !bLightOn;
			if (bLightOn) 
				glEnable(GL_LIGHTING);
			else
				glDisable(GL_LIGHTING);
			}	break;
#pragma region misc
		case 'a':
		case 'A':
			camera->moveState |= CAMERA_LEFT;
			break;
		case 's':
		case 'S':
			camera->moveState |= CAMERA_BACK;
			break;
		case 'd':
		case 'D':
			camera->moveState |= CAMERA_RIGHT;
			break;
		case 'w':
		case 'W':
			camera->moveState |= CAMERA_FORWARD;
			break;
		case 'r':
		case 'R':
			camera->moveState |= CAMERA_UP;
			break;
		case 'f':
		case 'F':
			camera->moveState |= CAMERA_DOWN;
			break;
		case 'g':
		case 'G':
			camera->groundMove = !camera->groundMove;
			break;
		
	case 27:
		cleanup();
		exit(0);
	}
	glutPostRedisplay();
}

void keyboardUpFunc(unsigned char key, int mx, int my){	
	switch(key){	
		case 'a':
		case 'A':
			camera->moveState &= ~CAMERA_LEFT;
			break;
		case 's':
		case 'S':
			camera->moveState &= ~CAMERA_BACK;
			break;
		case 'd':
		case 'D':
			camera->moveState &= ~CAMERA_RIGHT;
			break;
		case 'w':
		case 'W':
			camera->moveState &= ~CAMERA_FORWARD;
			break;
		case 'r':
		case 'R':
			camera->moveState &= ~CAMERA_UP;
			break;
		case 'f':
		case 'F':
			camera->moveState &= ~CAMERA_DOWN;
			break;
	}
	glutPostRedisplay();
}

void mouseFunc(int button, int state, int x, int y){
	//if(button == GLUT_LEFT_BUTTON && state == GLUT_DOWN)
}

void motionFunc(int x, int y){
	if(x != window.w/2 || y != window.h/2){
		camera->rotate((float)(window.w/2 - x), (float)(window.h/2 - y));
		glutWarpPointer(window.w/2, window.h/2);
	}
	glutPostRedisplay();
}

void cleanup(){
	delete camera;
	//delete texture;
	camera = NULL;
	//texture = NULL;
}
#pragma endregion
