/*
 *	NOTE: all vec4 operations work on xyz components 
 *  only. Manually divide by w if necessary. 
 *  -David
 */

#ifndef VEC4_H
#define VEC4_H

#include <iostream>
#include <math.h>
#include "defines.h"

typedef struct vec4{
	float v[4];

	vec4 &operator =(const vec4 &r){
		if (this != &r)
			memcpy(v, r.v, sizeof(float)*VEC4_SIZE);
		return *this;
	}

	vec4 &operator =(const float r[4]){
		if(v != r)
			memcpy(v, r, sizeof(float)*VEC4_SIZE);
		return *this;
	}

	float &operator [](int index){
		return v[index];
	}

	vec4 &operator +=(const vec4 &r){
		for(int i = 0; i < VEC4_SIZE-1; i++)
			v[i] += r.v[i];
		return *this;
	}

	vec4 operator +(const vec4 &r){
		return vec4(*this) += r;
	}

	vec4 &operator -=(const vec4 &r){
		for(int i = 0; i < VEC4_SIZE-1; i++)
			v[i] -= r.v[i];
		return *this;
	}	
	vec4 operator -(const vec4 &r){
		return vec4(*this) -= r;
	}

	vec4 &operator *=(float scalar){
		for(int i = 0; i < VEC4_SIZE-1; i++)
			v[i] *= scalar;
		return *this;
	}

	vec4 operator *(float scalar){
		return vec4(*this) *= scalar;
	}

	vec4 &operator /=(float scalar){
		for(int i = 0; i < VEC4_SIZE-1; i++)
			v[i] /= scalar;
		return *this;
	}

	vec4 operator /(float scalar){
		return vec4(*this) /= scalar;
	}

	operator float*(){
		return v;
	}

	float dp(const vec4 &r){
		float tot = 0;
		for(int i = 0; i < VEC4_SIZE-1; i++)
			tot += (v[i] * r.v[i]);
		return tot;
	}

	vec4 cross(const vec4 &r){
		return vec4(*this).crossEquals(r);
	}

	vec4 &crossEquals(const vec4 &r){
		for(int i = 0; i < VEC4_SIZE-1; i++)
			v[i] = v[(i+1)%3]*r.v[(i+2)%3] - v[(i+2)%3]*r.v[(i+1)%3];
		return *this;
	}

	float magnitude(){
		return sqrt(v[0]*v[0] + v[1]*v[1] + v[2]*v[2]);
	}

	vec4 normalize(){
		float distance = this->magnitude();
		for(int i = 0; i < VEC4_SIZE-1; i++)
			v[i] /= distance;
		return *this;
	}
}vec4;

extern std::ostream &operator <<(std::ostream &stream, vec4 &r);

#endif // VEC4_H