//varying vec3 ambient;
varying vec3 normal, viewVector, reflectVector;
varying vec3 lightDir1;
uniform vec4 lightColor; 
uniform float kRim, kR, kS,kSpec,  fresnelS;

uniform sampler2D modelTex;
uniform sampler2D skinTex;
const float C1 = 0.429043;
const float C2 = 0.511664;
const float C3 = 0.743125;
const float C4 = 0.886227;
const float C5 = 0.247708;

const vec3 L1m1 = vec3(0.175058, 0.245335, 0.312891);
const vec3 L11 = vec3(-0.004629, -0.029448, -0.048028);
const vec3 L2m2 = vec3(-0.120535, -0.121160, -0.117507);
const vec3 L2m1 = vec3(0.003242, 0.003624, 0.007511);
const vec3 L21 = vec3(-0.077539, -0.086325, -0.091591);
const vec3 L22 = vec3(-0.161784, -0.191783, -0.219152);
const vec3 L00 = vec3(0.871297, 0.875222, 0.864470);
const vec3 L20 = vec3(-0.028667, -0.024926, -0.020998);
const vec3 L10 = vec3(0.034675, 0.036107, 0.037362);

vec3 RGBToHSL(vec3 color)
{
	vec3 hsl; // init to 0 to avoid warnings ? (and reverse if + remove first part)
	
	float fmin = min(min(color.r, color.g), color.b);    //Min. value of RGB
	float fmax = max(max(color.r, color.g), color.b);    //Max. value of RGB
	float delta = fmax - fmin;             //Delta RGB value

	hsl.z = (fmax + fmin) / 2.0; // Luminance

	if (delta == 0.0)		//This is a gray, no chroma...
	{
		hsl.x = 0.0;	// Hue
		hsl.y = 0.0;	// Saturation
	}
	else                                    //Chromatic data...
	{
		if (hsl.z < 0.5)
			hsl.y = delta / (fmax + fmin); // Saturation
		else
			hsl.y = delta / (2.0 - fmax - fmin); // Saturation
		
		float deltaR = (((fmax - color.r) / 6.0) + (delta / 2.0)) / delta;
		float deltaG = (((fmax - color.g) / 6.0) + (delta / 2.0)) / delta;
		float deltaB = (((fmax - color.b) / 6.0) + (delta / 2.0)) / delta;

		if (color.r == fmax )
			hsl.x = deltaB - deltaG; // Hue
		else if (color.g == fmax)
			hsl.x = (1.0 / 3.0) + deltaR - deltaB; // Hue
		else if (color.b == fmax)
			hsl.x = (2.0 / 3.0) + deltaG - deltaR; // Hue

		if (hsl.x < 0.0)
			hsl.x += 1.0; // Hue
		else if (hsl.x > 1.0)
			hsl.x -= 1.0; // Hue
	}

	return hsl;
}

float HueToRGB(float f1, float f2, float hue)
{
	if (hue < 0.0)
		hue += 1.0;
	else if (hue > 1.0)
		hue -= 1.0;
	float res;
	if ((6.0 * hue) < 1.0)
		res = f1 + (f2 - f1) * 6.0 * hue;
	else if ((2.0 * hue) < 1.0)
		res = f2;
	else if ((3.0 * hue) < 2.0)
		res = f1 + (f2 - f1) * ((2.0 / 3.0) - hue) * 6.0;
	else
		res = f1;
	return res;
}
vec3 HSLToRGB(vec3 hsl)
{
	vec3 rgb;
	
	if (hsl.y == 0.0)
		rgb = vec3(hsl.z); // Luminance
	else
	{
		float f2;
		
		if (hsl.z < 0.5)
			f2 = hsl.z * (1.0 + hsl.y);
		else
			f2 = (hsl.z + hsl.y) - (hsl.y * hsl.z);
			
		float f1 = 2.0 * hsl.z - f2;
		
		rgb.r = HueToRGB(f1, f2, hsl.x + (1.0/3.0));
		rgb.g = HueToRGB(f1, f2, hsl.x);
		rgb.b= HueToRGB(f1, f2, hsl.x - (1.0/3.0));
	}
	
	return rgb;
}

vec3 BlendLuminosity(vec3 base, vec3 blend)
{
	vec3 baseHSL = RGBToHSL(base);
	return HSLToRGB(vec3(baseHSL.r, baseHSL.g, RGBToHSL(blend).b));
}

#define BlendDarkenf(base, blend) 		min(blend, base)
#define BlendDarken				BlendDarkenf

varying vec2 texCoord;
void main(void)
{
   vec3 n;
   float NdotL1,NdotV,NdotU,VdotR,fresnelR,warpFun;
   vec4 albedo;
   vec3 ambient;
   
   vec4 color = vec4(0.0,0.0,0.0,0.0);
   
   n = normalize(normal);
   ambient = 0.1 * (C1 * L22 * (n.x * n.x - n.y * n.y) +
             C3 * L20 * (n.z * n.z) +
             C4 * L00 +
             C5 * L20 +
             2.0 * C1 * L2m2 * n.x * n.y +
             2.0 * C1 * L21 * n.x * n.z +
             2.0 * C1 * L2m1 * n.y * n.z +
             2.0 * C2 * L11 * n.x +
             2.0 * C2 * L1m1 * n.y +
             2.0 * C2 * L10 * n.z);
   NdotL1 = max(dot(n,lightDir1),0.0);
   NdotV = dot(n,viewVector);
   fresnelR = 1.0 - pow(NdotV, 4.0);
   NdotU = dot(n, vec3(0.0,1.0,0.0));
   VdotR = dot(viewVector, reflectVector);
   albedo = texture2D(modelTex, texCoord);
   color += vec4(ambient,1.0);
   //color += vec4(0.1,0.1,0.1,1.0);
   
   //if(NdotL1>0.0)
   {   
		warpFun = pow(0.5*NdotL1 + 0.5 , 2.0);
		color += clamp(texture2D(skinTex,vec2(warpFun,0.0)) * 2.0, 0.0,1.0);      
   }
   
   if(VdotR<0.0) VdotR=0.0;
   color += NdotU * fresnelR * kR * vec4(ambient,1.0);
   color += lightColor * kS *max(fresnelS*pow(VdotR,kSpec), fresnelR*kR*pow(VdotR,kRim));
   
   //color *= albedo;
   
   albedo += NdotU * fresnelR * kR * vec4(ambient,1.0);
   albedo += lightColor * kS *max(fresnelS*pow(VdotR,kSpec), fresnelR*kR*pow(VdotR,kRim));  
   //color = vec4( BlendLuminosity(albedo.xyz, color.xyz), 1);

   color = BlendDarken(color, albedo) ;
   	
   gl_FragColor = color;
}