uniform sampler2D colorTex; //256x256
uniform sampler2D paperTex; //256x256
uniform sampler2D granTex;  //256x256
uniform sampler2D sceneTex;
//uniform vec4 color;
uniform vec2 fViewportDimensions;
uniform float M, D, G, P, S;

varying float mapCoord;
varying vec2 texCoord;

vec4 sobelColor(vec2 coord){
   vec4 screenColor = texture2D(paperTex, coord);
   vec4 granColor = texture2D(granTex, coord);
   vec4 texColor = texture2D(sceneTex, texCoord);
   return /*color*/ texColor - (screenColor * granColor * G);
}

vec4 sobel(vec2 tCoord){
   float offset = 1.0/256.0;
   vec2 coord = tCoord;
   vec4 color00, color01, color02,
        color10, color11, color12,
        color20, color21, color22;
   vec4 sobelX, sobelY;
   
   coord = vec2(tCoord.x-offset, tCoord.y-offset);
   color00 = sobelColor(coord);
   coord = vec2(tCoord.x, tCoord.y-offset);
   color01 = sobelColor(coord);
   coord = vec2(tCoord.x+offset, tCoord.y-offset);
   color02 = sobelColor(coord);
   coord = vec2(tCoord.x-offset, tCoord.y);
   color10 = sobelColor(coord);
   coord = vec2(tCoord.x, tCoord.y);
   color11 = sobelColor(coord);
   coord = vec2(tCoord.x+offset, tCoord.y);
   color12 = sobelColor(coord);
   coord = vec2(tCoord.x-offset, tCoord.y+offset);
   color20 = sobelColor(coord);
   coord = vec2(tCoord.x, tCoord.y+offset);
   color21 = sobelColor(coord);
   coord = vec2(tCoord.x+offset, tCoord.y+offset);
   color22 = sobelColor(coord);
   
   sobelX = -color00 - 2.0*color01 - color02 +
            color20 + 2.0*color21 + color22;
   sobelY = -color00 - 2.0*color10 - color20 +
            color02 + 2.0*color12 + color22;
            
   return abs(sobelX) + abs(sobelY);
}

void main( void )
{
   vec4 one = vec4(1.0, 1.0, 1.0, 1.0);
   
   vec2 screenTexCoord = gl_FragCoord.xy/fViewportDimensions;
   float colormapCoord = mapCoord + ((texture2D(paperTex, texCoord*4).x-M/2.0)*D);
   colormapCoord = clamp(colormapCoord, 0.01, .99);
   
   vec4 texColor = texture2D(sceneTex, texCoord);
   vec4 colormapColor  = texture2D(colorTex, vec2(colormapCoord, 0.0));
   vec4 orgColor = texColor * colormapColor;
   vec4 paperColor = texture2D(paperTex, texCoord*4);
   
   gl_FragColor = orgColor - paperColor*P - sobel(vec2(colormapCoord, 0.0))*S;
}